@@ -47,21 +47,38 @@ func list(g *gt.GopathTest, c *Context, name, expected string) {
 		output.WriteString(statusItemString(item))
 		output.WriteRune('\n')
 	}
-	if !stringSameIgnoreSpace(output.String(), expected) {
-		g.Fatalf("(%s) Got\n%s", name, output.String())
+	if ok, msg := stringSameIgnoreSpace(output.String(), expected); !ok {
+		g.Fatalf("%s(%s) Got\n%s", msg, name, output.String())
 	}
 }
 
-func stringSameIgnoreSpace(a, b string) bool {
+func stringSameIgnoreSpace(a, b string, ignore ...string) (bool, string) {
 	// Remove any space padding on the start/end of each line.
-	trimLines := func(s string) string {
-		lines := strings.Split(strings.TrimSpace(s), "\n")
-		for i := range lines {
-			lines[i] = strings.TrimSpace(lines[i])
-		}
-		return strings.Join(lines, "\n")
+	a = strings.TrimSpace(a)
+	b = strings.TrimSpace(b)
+	aLine := strings.Split(a, "\n")
+	bLine := strings.Split(b, "\n")
+	ct := len(aLine)
+	if len(bLine) < ct {
+		ct = len(bLine)
 	}
-	return trimLines(a) == trimLines(b)
+lineLoop:
+	for i := 0; i < ct; i++ {
+		al := strings.TrimSpace(aLine[i])
+		bl := strings.TrimSpace(bLine[i])
+		if al != bl {
+			for _, ignorePrefix := range ignore {
+				if strings.HasPrefix(al, ignorePrefix) && strings.HasPrefix(bl, ignorePrefix) {
+					continue lineLoop
+				}
+			}
+			return false, fmt.Sprintf("A: %s\nB: %s\n", al, bl)
+		}
+	}
+	if len(aLine) != len(bLine) {
+		return false, "Different Line Count"
+	}
+	return true, ""
 }
 
 func verifyChecksum(g *gt.GopathTest, c *Context, name string) {
@@ -103,14 +120,14 @@ func statusItemString(li StatusItem) string {
 	return fmt.Sprintf("%s %s [%s] < %q", li.Status.String(), li.Local, li.Pkg.Path, li.ImportedBy)
 }
 
-func vendorFile(g *gt.GopathTest, name, expected string) {
+func vendorFile(g *gt.GopathTest, name, expected string, ignore ...string) {
 	buf, err := ioutil.ReadFile(filepath.Join(g.Current(), relVendorFile))
 	if err != nil {
 		g.Fatal(err)
 	}
 	s := string(buf)
-	if !stringSameIgnoreSpace(s, expected) {
-		g.Fatalf("(%s) Got:\n%s\nWant\n%s", name, s, expected)
+	if ok, msg := stringSameIgnoreSpace(s, expected, ignore...); !ok {
+		g.Fatalf("(%s) \n%sGot:\n%s\nWant\n%s", name, msg, s, expected)
 	}
 }
 
@@ -465,7 +482,7 @@ pv  co1/vendor/co2/pk1 [co2/pk1] < []
  s  testing < ["co1/vendor/co2/pk1"]
 `)
 
-	c.IgnoreBuild("test")
+	c.IgnoreBuildAndPackage("test")
 
 	list(g, c, "ignore test", `
 pv  co1/vendor/co2/pk1 [co2/pk1] < []
@@ -936,7 +953,7 @@ func TestTagList(t *testing.T) {
  s  encoding/json < ["co2/pk1"]
  s  testing < ["co1/pk1" "co2/pk1"]
 `)
-	c.IgnoreBuild("test appengine")
+	c.IgnoreBuildAndPackage("test appengine")
 
 	list(g, c, "co1 list", `
  e  co2/pk1 < ["co1/pk1"]
@@ -947,7 +964,7 @@ func TestTagList(t *testing.T) {
  s  testing < ["co1/pk1"]
 `)
 
-	c.IgnoreBuild("")
+	c.IgnoreBuildAndPackage("")
 
 	g.Check(c.ModifyStatus(StatusGroup{
 		Status: []Status{{Location: LocationExternal}},
@@ -964,7 +981,7 @@ func TestTagList(t *testing.T) {
  s  encoding/json < ["co1/vendor/co2/pk1"]
  s  testing < ["co1/pk1" "co1/vendor/co2/pk1"]
 `)
-	c.IgnoreBuild("test appengine")
+	c.IgnoreBuildAndPackage("test appengine")
 
 	list(g, c, "after co1 list", `
  v  co1/vendor/co2/pk1 [co2/pk1] < ["co1/pk1"]
@@ -994,7 +1011,7 @@ func TestTagAdd(t *testing.T) {
 	)
 	g.In("co1")
 	c := ctx(g)
-	c.IgnoreBuild("test appengine")
+	c.IgnoreBuildAndPackage("test appengine")
 
 	g.Check(c.ModifyImport(pkg("co2/pk1"), Add))
 	g.Check(c.Alter())
@@ -1372,3 +1389,98 @@ func TestRelativePath(t *testing.T) {
  s  strings < ["co2/pk1"]
 `)
 }
+
+func TestAddTreeWithSamePrefix(t *testing.T) {
+	g := gt.New(t)
+	defer g.Clean()
+
+	g.Setup("co1/pk1",
+		gt.File("a.go"),
+	)
+	g.Setup("co2/pk1",
+		gt.File("a.go"),
+	)
+	g.Setup("co2/pk1-1",
+		gt.File("a.go"),
+	)
+	g.In("co1")
+	c := ctx(g)
+	g.Check(c.ModifyImport(pkg("co2/pk1"), Add, IncludeTree))
+	g.Check(c.Alter())
+	g.Check(c.ModifyImport(pkg("co2/pk1-1"), Add, IncludeTree))
+	g.Check(c.Alter())
+}
+
+func TestAddTreeWithSamePrefix2(t *testing.T) {
+	g := gt.New(t)
+	defer g.Clean()
+
+	g.Setup("co1/pk1",
+		gt.File("a.go"),
+	)
+	g.Setup("co2/pk1",
+		gt.File("a.go"),
+	)
+	g.Setup("co2/pk1-1",
+		gt.File("a.go"),
+	)
+	g.In("co1")
+	c := ctx(g)
+	g.Check(c.ModifyImport(pkg("co2/pk1-1"), Add, IncludeTree))
+	g.Check(c.Alter())
+	g.Check(c.ModifyImport(pkg("co2/pk1"), Add, IncludeTree))
+	g.Check(c.Alter())
+}
+
+func TestOrginRetain(t *testing.T) {
+	g := gt.New(t)
+	defer g.Clean()
+
+	g.Setup("co1/pk1",
+		gt.File("a.go"),
+	)
+	g.Setup("co2/pk2",
+		gt.File("a.go"),
+	)
+	g.In("co1")
+	c := ctx(g)
+	g.Check(c.ModifyImport(pkg("correct/name/pk2::co2/pk2"), Add))
+	g.Check(c.Alter())
+	g.Check(c.WriteVendorFile())
+
+	vendorFile(g, "after add", `
+{
+	"comment": "",
+	"ignore": "",
+	"package": [
+		{
+			"checksumSHA1": "KcwRyEXPUn2jwAgWGhFDjIt3deI=",
+			"origin": "co2/pk2",
+			"path": "correct/name/pk2",
+			"revision": ""
+		}
+	],
+	"rootPath": "co1"
+}
+`)
+
+	g.Check(c.ModifyImport(pkg("correct/name/pk2"), Update))
+	g.Check(c.Alter())
+	g.Check(c.WriteVendorFile())
+
+	vendorFile(g, "after update", `
+{
+	"comment": "",
+	"ignore": "",
+	"package": [
+		{
+			"checksumSHA1": "KcwRyEXPUn2jwAgWGhFDjIt3deI=",
+			"origin": "co2/pk2",
+			"path": "correct/name/pk2",
+			"revision": ""
+		}
+	],
+	"rootPath": "co1"
+}
+`)
+}
