@@ -11,7 +11,6 @@ import (
 	"bytes"
 	"crypto/sha1"
 	"encoding/base64"
-	"errors"
 	"fmt"
 	"math"
 	"path"
@@ -24,6 +23,7 @@ import (
 	"github.com/kardianos/govendor/pkgspec"
 	"github.com/kardianos/govendor/vcs"
 	"github.com/kardianos/govendor/vendorfile"
+	"github.com/pkg/errors"
 )
 
 // OperationState is the state of the given package move operation.
@@ -129,6 +129,10 @@ statusLoop:
 		if ctx.added[item.Pkg.PathOrigin()] {
 			continue
 		}
+		// Do not add excluded packages
+		if item.Status.Presence == PresenceExcluded {
+			continue
+		}
 		// Do not attempt to add any existing status items that are
 		// already present in vendor folder.
 		if mod == Add {
@@ -163,6 +167,17 @@ func (ctx *Context) ModifyImport(imp *pkgspec.Pkg, mod Modify, mops ...ModifyOpt
 	if ctx.added == nil {
 		ctx.added = make(map[string]bool, 10)
 	}
+	// Grap the origin of the pkg spec from the vendor file as needed.
+	if len(imp.Origin) == 0 {
+		for _, vpkg := range ctx.VendorFile.Package {
+			if vpkg.Remove {
+				continue
+			}
+			if vpkg.Path == imp.Path {
+				imp.Origin = vpkg.Origin
+			}
+		}
+	}
 	if !imp.MatchTree {
 		if !ctx.added[imp.PathOrigin()] {
 			err = ctx.modify(imp, mod, mops)
@@ -221,8 +236,8 @@ func (ctx *Context) modify(ps *pkgspec.Pkg, mod Modify, mops []ModifyOption) err
 	}
 	tree := ps.IncludeTree
 
-	// Determine if we can find the source path from an add or update.
 	switch mod {
+	// Determine if we can find the source path from an add or update.
 	case Add, Update, AddUpdate:
 		_, _, err = ctx.findImportDir("", ps.PathOrigin())
 		if err != nil {
@@ -259,7 +274,9 @@ func (ctx *Context) modify(ps *pkgspec.Pkg, mod Modify, mops []ModifyOption) err
 		pkg.Origin = ps.PathOrigin()
 		pkg.Path = ps.Path
 	}
-	if ps.Origin != pkg.Origin {
+
+	pkg.HasOrigin = ps.HasOrigin
+	if ps.HasOrigin {
 		pkg.Origin = ps.Origin
 	}
 
@@ -391,7 +408,10 @@ func (ctx *Context) modifyAdd(pkg *Package, uncommitted bool) error {
 	if pkg.IncludeTree {
 		vp.Tree = pkg.IncludeTree
 	}
-	vp.Origin = pkg.Origin
+
+	if pkg.HasOrigin {
+		vp.Origin = pkg.Origin
+	}
 	if pkg.Path != pkg.Local && pkg.inVendor && vp.Add {
 		vp.Origin = pkg.Local
 	}
@@ -495,10 +515,13 @@ func (ctx *Context) modifyFetch(pkg *Package, uncommitted, hasVersion bool, vers
 	}
 	if hasVersion {
 		vp.Version = version
+		pkg.Version = version
+		pkg.HasVersion = true
 	}
 	if pkg.IncludeTree {
 		vp.Tree = pkg.IncludeTree
 	}
+	pkg.Origin = strings.TrimPrefix(pkg.Origin, ctx.RootImportPath+"/"+ctx.VendorFolder+"/")
 	vp.Origin = pkg.Origin
 	origin := vp.Origin
 	if len(vp.Origin) == 0 {
@@ -691,7 +714,7 @@ func (ctx *Context) Alter() error {
 				}
 			}
 			if err != nil {
-				return fmt.Errorf("Failed to fetch package %q: %v", op.Pkg.Path, err)
+				return errors.Wrapf(err, "Failed to fetch package %q", op.Pkg.Path)
 			}
 		}
 		if len(nextOps) == 0 {
@@ -719,10 +742,14 @@ func (ctx *Context) Alter() error {
 			err = RemovePackage(op.Src, filepath.Join(ctx.RootDir, ctx.VendorFolder), pkg.IncludeTree)
 			op.State = OpDone
 		case OpCopy:
-			ctx.copyOperation(op, nil)
+			err = ctx.copyOperation(op, nil)
+			if os.IsNotExist(errors.Cause(err)) {
+				// Ignore packages that don't exist, like appengine.
+				err = nil
+			}
 		}
 		if err != nil {
-			return fmt.Errorf("Failed to %v package %q -> %q: %v", op.Type, op.Src, op.Dest, err)
+			return errors.Wrapf(err, "Failed to %v package %q -> %q", op.Type, op.Src, op.Dest)
 		}
 	}
 	if ctx.rewriteImports {
@@ -750,7 +777,7 @@ func (ctx *Context) copyOperation(op *Operation, beforeCopy func(deps []string)
 	}
 	op.State = OpDone
 	if err != nil {
-		return fmt.Errorf("copy failed. dest: %q, src: %q, pkgPath %q, err %v", op.Dest, op.Src, root, err)
+		return errors.Wrapf(err, "copy failed. dest: %q, src: %q, pkgPath %q", op.Dest, op.Src, root)
 	}
 	return nil
 }
